#! /usr/bin/env python
# -.- coding: utf-8 -.-
#
# Sezen
#
# Copyright © 2009-2010 Seif Lotfy <seif@lotfy.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import time
import gobject
import os
import urllib

from xdg.Config import icon_size
from zeitgeist.client import ZeitgeistClient
from zeitgeist.datamodel import Event, Subject, Interpretation, Manifestation, \
	ResultType, TimeRange

try:
	CLIENT = ZeitgeistClient()
except RuntimeError, e:
	print "Unable to connect to Zeitgeist: %s" % e
	CLIENT = None
	
import dbus

_bus = dbus.SessionBus()
_fts = _bus.get_object('org.gnome.zeitgeist.Engine',
                      '/org/gnome/zeitgeist/index/activity')
fts = dbus.Interface(_fts, 'org.gnome.zeitgeist.Index')



SUPPORTED_SOURCES = {
	# TODO: Please change this description stuff. To give a single reason,
	# saying "videos watched" is a lie because you may have edited them, or
	# it may just be a "bookmark event".
	#
	# Also, how this is implemented now won't work fine with i18n.	
	#"sezen://home": {"name": "Home", "icon":"gtk-home", "uri": "sezen://home"},
	"all": {"name": "All", "icon":"", "uri": "all"},
	
	Interpretation.AUDIO.uri: {"name": "Audio", "icon":"gnome-mime-audio", "uri": Interpretation.AUDIO},
	Interpretation.DOCUMENT.uri: {"name": "Documents", "icon":"x-office-document", "uri": Interpretation.DOCUMENT},
	#Interpretation.SOURCE_CODE.uri: {"name": "Development", "icon":"stock_new-presentation", "uri": Interpretation.SOURCE_CODE},
	Interpretation.IMAGE.uri: {"name": "Images", "icon":"image", "uri": Interpretation.IMAGE},
	Interpretation.VIDEO.uri: {"name": "Video", "icon":"gnome-mime-video", "uri": Interpretation.VIDEO},
	Interpretation.WEBSITE.uri: {"name": "Websites", "icon":"gnome-globe", "uri": Interpretation.WEBSITE},
	#Interpretation.IMMESSAGE.uri: {"name": "People", "uri":Interpretation.IMMESSAGE, "icon":"stock_person"}
}
	
class DataSink(gobject.GObject):
	__gsignals__ = {
		"reload" : (gobject.SIGNAL_RUN_FIRST,
					gobject.TYPE_NONE,())}

	def __init__(self):
		gobject.GObject.__init__(self)
		self.zg = CLIENT
		self.events = []
		self.last_event = None
		
		def notify_insert_handler(time_range, events):
			if self.last_event != events[0].subjects[0].uri:
				self.last_event = events[0].subjects[0].uri
				self.load_events()
		
		events = []
		self.zg.install_monitor(TimeRange.always(), events,
			notify_insert_handler, notify_insert_handler)
		
	def get_sources(self):
		return SUPPORTED_SOURCES

	def load_events(self):

		t = time.time()
		
		def _handle_find_events(events):
			print "*** events:", len(events),"     time: ", time.time() - t
			self.events = events
			self.emit("reload")
				
		events = []
		self.zg.find_events_for_templates([], _handle_find_events,
			TimeRange.until_now(), num_events=0	, result_type=2)
		

	def search(self, text, callback):
		if not text.strip() == "":	
			text = text.lower()
			events = []
			
			results, count = fts.Search(text, TimeRange.always(), [], 
				0, 0, 2)
			print "RESULTS", results
			results = map(Event, results)
			
			
			for event in self.events:
				subject = event.subjects[0]
				if text in subject.text.lower() or text in subject.uri.lower():
					events.append(event)
			callback(events)
		else:
			callback(self.events)
